@@ -44,6 +44,7 @@ USER_CONFIG = os.path.join(CONFIGDIR, "git-review.conf")
 DEFAULTS = dict(scheme='ssh', hostname=False, port=None, project=False,
                 branch='master', remote="gerrit", rebase="1",
                 track="0", usepushurl="0", notopic=False, branchauthor="name")
+LOCAL_GIT_VERSION = (0, 0, 0)
 COPYRIGHT = """\
 Copyright OpenStack Foundation and OpenDev Contributors
 
@@ -62,6 +63,10 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+# Bump this if a feature of a more recent git is unconditionally
+# required
+MINIMUM_GIT_VERSION = (2, 10, 0)
+
 
 _branch_name = None
 _has_color = None
@@ -94,7 +99,7 @@ class CommandFailed(GitReviewException):
             ("output", self.output)])
 
     def __str__(self):
-        return self.__doc__ + """
+        return """
 The following command failed with exit code %(rc)d
     "%(argv)s"
 -----------------------
@@ -109,7 +114,7 @@ class ChangeSetException(GitReviewException):
         self.e = str(e)
 
     def __str__(self):
-        return self.__doc__ % self.e
+        return self.e
 
 
 def printwrap(unwrapped):
@@ -219,6 +224,29 @@ def get_version():
     return provider.version
 
 
+def get_git_version():
+    global LOCAL_GIT_VERSION
+    output = run_command("git version")
+    if "git version" in output:
+        try:
+            # Git version on Linux is of the form:
+            #   git version 2.35.3
+            # But on OS X we get:
+            #   git version 2.20.1 (Apple Git-117)
+            # Keep this as simple as possible by splitting on whitespace
+            # and then selecting the 3rd element which should be the version.
+            v = output.split()[2]
+            LOCAL_GIT_VERSION = tuple(map(int, v.split('.')[:3]))
+        except Exception:
+            printwrap("Could not determine git version!")
+            sys.exit(1)
+    if LOCAL_GIT_VERSION < MINIMUM_GIT_VERSION:
+        printwrap("Local git version %s < required git version %s" %
+                  '.'.join(map(str, LOCAL_GIT_VERSION)),
+                  '.'.join(map(str, MINIMUM_GIT_VERSION)))
+        sys.exit(1)
+
+
 def git_directories():
     """Determine (absolute git work directory path, .git subdirectory path)."""
     cmd = ("git", "rev-parse", "--show-toplevel", "--git-dir")
@@ -373,9 +401,17 @@ def set_hooks_commit_msg(remote, target_file):
                 userhost = hostname
             # OS independent target file
             scp_target_file = target_file.replace(os.sep, "/")
+
+            # Get scp options
+            scp_out = run_command("scp")
+            scp_opts = scp_out[scp_out.index("[") + 2:scp_out.index("]")]
+
             cmd = ["scp", userhost + ":hooks/commit-msg", scp_target_file]
             if port is not None:
                 cmd.insert(1, "-P%s" % port)
+            # Force scp protocol if the -O option is available
+            if "O" in scp_opts:
+                cmd.insert(1, "-O")
 
             if VERBOSE:
                 hook_url = 'scp://%s%s/hooks/commit-msg' \
@@ -909,20 +945,9 @@ def rebase_changes(branch, remote, interactive=True):
                   "re-run with the '-R' option enabled." % (branch, remote))
         sys.exit(1)
 
-    # Determine git version to set rebase flags below.
-    output = run_command("git version")
-    rebase_flag = "--rebase-merges"
-    if "git version" in output:
-        try:
-            v = output.rsplit(None, 1)[1]
-            gitv = tuple(map(int, v.split('.')[:3]))
-            if gitv < (2, 18, 0):
-                rebase_flag = "--preserve-merges"
-        except Exception:
-            # We tried to determine the version and failed. Use current git
-            # flag as fallback.
-            warn("Could not determine git version. "
-                 "Using modern git rebase flags.")
+    rebase_flag = '--rebase-merges'
+    if LOCAL_GIT_VERSION < (2, 18, 0):
+        rebase_flag = "--preserve-merges"
 
     interactive_flag = interactive and '-i' or ''
 
@@ -976,8 +1001,8 @@ def assert_one_change(remote, branch, yes, have_hook):
         use_color = "--color=always"
     else:
         use_color = "--color=never"
-    cmd = ("git log %s --decorate --oneline HEAD --not --remotes=%s" % (
-           use_color, remote))
+    cmd = ("git log %s --decorate --oneline --no-show-signature "
+           "HEAD --not --remotes=%s" % (use_color, remote))
     (status, output) = run_command_status(cmd)
     if status != 0:
         print("Had trouble running %s" % cmd)
@@ -1005,7 +1030,10 @@ def assert_one_change(remote, branch, yes, have_hook):
                       "branches (for independent changes).")
             print("\nThe outstanding commits are:\n\n%s\n\n"
                   "Do you really want to submit the above commits?" % output)
-            yes_no = input("Type 'yes' to confirm, other to cancel: ")
+            try:
+                yes_no = input("Type 'yes' to confirm, other to cancel: ")
+            except KeyboardInterrupt:
+                yes_no = "no"
             if yes_no.lower().strip() != "yes":
                 print("Aborting.")
                 sys.exit(1)
@@ -1666,6 +1694,8 @@ additional information:
         branch = options.branch
         options.track = False
 
+    get_git_version()
+
     global VERBOSE
     global UPDATE
     VERBOSE = options.verbose
