@@ -18,6 +18,7 @@
 import json
 import os
 import shutil
+import tempfile
 import testtools
 
 from git_review import tests
@@ -102,6 +103,53 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         self._run_git_review(
             '-s', chdir=os.path.join(self.test_dir, 'subdirectory'))
 
+    def test_git_review_s_without_core_hooks_path_option(self):
+        """Test whether git-review -s correctly creates the commit-msg hook,
+        with the Git core.hooksPath option unset.
+        """
+        hooks_subdir = ".git/hooks"
+        self.reset_remote()
+
+        # There really isn't a good way to ensure that
+        # "core.hooksPath" option is presently unset. "git config
+        # --unset" is not idempotent; if you try to unset a config
+        # option that isn't defined it fails with an exit code of
+        # 5. Running "git config core.hooksPath" to retrieve the value
+        # returns 1 if unset, but we can't use self.assertRaises here
+        # either because run_cmd raises a generic Exception and that's
+        # too broad. So instead, we don't check core.hooksPath at all
+        # here, and instead rely on the next two tests to unset the
+        # option after they've set it.
+        self._run_git_review('-s')
+        self.assertTrue(os.path.exists(os.path.join(self.test_dir,
+                                                    hooks_subdir,
+                                                    'commit-msg')))
+
+    def test_git_review_s_with_core_hooks_path_option_relative(self):
+        """Test whether git-review -s correctly creates the commit-msg hook,
+        with the Git core.hooksPath option set to a relative path.
+        """
+        hooks_subdir = "foo"
+        self.reset_remote()
+        self._run_git("config", "core.hooksPath", hooks_subdir)
+        self._run_git_review('-s')
+        self.assertTrue(os.path.exists(os.path.join(self.test_dir,
+                                                    hooks_subdir,
+                                                    'commit-msg')))
+        self._run_git("config", "--unset", "core.hooksPath")
+
+    def test_git_review_s_with_core_hooks_path_option_absolute(self):
+        """Test whether git-review -s correctly creates the commit-msg hook,
+        with the Git core.hooksPath option set to an absolute path.
+        """
+        self.reset_remote()
+        with tempfile.TemporaryDirectory() as hooks_dir:
+            self._run_git("config", "core.hooksPath", hooks_dir)
+            self._run_git_review('-s')
+            self.assertTrue(os.path.exists(os.path.join(hooks_dir,
+                                                        'commit-msg')))
+        self._run_git("config", "--unset", "core.hooksPath")
+
     def test_git_review_d(self):
         """Test git-review -d."""
         self._run_git_review('-s')
@@ -222,6 +270,38 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
 
         self.assertEqual(set(['reviewer1', 'reviewer2']), reviewers)
 
+    def test_rebase_unstaged_changes_msg(self):
+        """Test message displayed when unstaged changes are present."""
+        self._run_git_review('-s')
+        self._run_git('checkout', '-b', 'test_branch')
+        self._unstaged_change(change_text='simple message')
+        exc = self.assertRaises(Exception, self._run_git_review)
+        self.assertIn("You have unstaged changes. Please", exc.args[0])
+
+    def test_rebase_uncommitted_changes_msg(self):
+        """Test message displayed when staged changes are present."""
+        self._run_git_review('-s')
+        self._run_git('checkout', '-b', 'test_branch')
+        self._uncommitted_change(change_text='simple message')
+        exc = self.assertRaises(Exception, self._run_git_review)
+        self.assertIn("You have uncommitted changes. Please", exc.args[0])
+
+    def test_ignore_unstaged_submodule_changes(self):
+        """Test message displayed when unstaged changes are present."""
+        self._run_git_review('-s')
+        self._run_git('checkout', '-b', 'test_branch')
+        self._run_git_sub('init')
+        self._unstaged_change_sub(change_text='simple message')
+        self._run_git_review()
+
+    def test_ignore_uncommitted_submodule_changes(self):
+        """Test message displayed when staged changes are present."""
+        self._run_git_review('-s')
+        self._run_git('checkout', '-b', 'test_branch')
+        self._run_git_sub('init')
+        self._uncommitted_change_sub(change_text='simple message')
+        self._run_git_review()
+
     def test_rebase_no_remote_branch_msg(self):
         """Test message displayed where no remote branch exists."""
         self._run_git_review('-s')
@@ -243,9 +323,11 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
                             'create conflict with master')
 
         exc = self.assertRaises(Exception, self._run_git_review)
-        self.assertIn(
-            "Errors running git rebase -p -i remotes/%s/master" % self._remote,
-            exc.args[0])
+        rebased = ("Errors running git rebase --rebase-merges "
+                   "-i remotes/%s/master" % self._remote in exc.args[0] or
+                   "Errors running git rebase --preserve-merges "
+                   "-i remotes/%s/master" % self._remote in exc.args[0])
+        self.assertTrue(rebased)
         self.assertIn("It is likely that your change has a merge conflict.",
                       exc.args[0])
 
@@ -261,9 +343,11 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
                             self._dir('test', 'new_test_file.txt'))
 
         review_res = self._run_git_review('-v')
-        self.assertIn(
-            "Running: git rebase -p -i remotes/%s/master" % self._remote,
-            review_res)
+        rebased = ("Running: git rebase --rebase-merges "
+                   "-i remotes/%s/master" % self._remote in review_res or
+                   "Running: git rebase --preserve-merges "
+                   "-i remotes/%s/master" % self._remote in review_res)
+        self.assertTrue(rebased)
         self.assertEqual(self._run_git('rev-parse', 'HEAD^1'), head_1)
 
     def test_uploads_with_nondefault_rebase(self):
@@ -287,8 +371,8 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
 
         br_out = self._run_git('checkout',
                                '-b', 'test_branch', 'origin/maint')
-        expected_track = ".*\nBranch '?test_branch'? set up to track remote" + \
-                         " branch '?maint'? from '?origin'?."
+        expected_track = (".*\nBranch '?test_branch'? set up to track remote "
+                          "branch '?maint'? from '?origin'?.")
         track_matcher = testtools.matchers.MatchesRegex(expected_track)
         self.assertThat(br_out, track_matcher)
         branches = self._run_git('branch', '-a')
@@ -303,8 +387,10 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
 
         review_res = self._run_git_review('-v')
         # no rebase needed; if it breaks it would try to rebase to master
-        self.assertNotIn("Running: git rebase -p -i remotes/origin/master",
-                         review_res)
+        self.assertNotIn("Running: git rebase --rebase-merges "
+                         "-i remotes/origin/master", review_res)
+        self.assertNotIn("Running: git rebase --preserve-merges "
+                         "-i remotes/origin/master", review_res)
         # Don't need to query gerrit for the branch as the second half
         # of this test will work only if the branch was correctly
         # stored in gerrit
@@ -321,8 +407,11 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         self.assertEqual(change_id, new_change_id)
         review_res = self._run_git_review('-v')
         # caused the right thing to happen
-        self.assertIn("Running: git rebase -p -i remotes/origin/maint",
-                      review_res)
+        rebased = ("Running: git rebase --rebase-merges "
+                   "-i remotes/origin/maint" in review_res or
+                   "Running: git rebase --preserve-merges "
+                   "-i remotes/origin/maint" in review_res)
+        self.assertTrue(rebased)
 
         # track different branch than expected in changeset
         branch = self._run_git('rev-parse', '--abbrev-ref', 'HEAD')
@@ -403,6 +492,11 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         self._assert_branch_would_be('master%topic=zat',
                                      extra_args=['-t', 'zat'])
 
+        # -t takes precedence over notopic
+        self._run_git('config', 'gitreview.notopic', 'true')
+        self._assert_branch_would_be('master%topic=zat',
+                                     extra_args=['-t', 'zat'])
+
     def test_git_review_T(self):
         self._run_git_review('-s')
         self._run_git('checkout', '-b', 'bug/456')
@@ -410,6 +504,14 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         self._assert_branch_would_be('master%topic=bug/456')
         self._assert_branch_would_be('master', extra_args=['-T'])
 
+        self._run_git('config', 'gitreview.notopic', 'true')
+        self._assert_branch_would_be('master')
+        self._run_git('config', 'gitreview.notopic', 'false')
+        self._assert_branch_would_be('master%topic=bug/456')
+
+        # -T takes precedence over notopic=false
+        self._assert_branch_would_be('master', extra_args=['-T'])
+
     def test_git_review_T_t(self):
         self.assertRaises(Exception, self._run_git_review, '-T', '-t', 'taz')
 
@@ -427,8 +529,8 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         self._run_gerrit_cli('review', head, '--abandon')
 
         # Populate "project2" repo
-        self._run_gerrit_cli('create-project', '--empty-commit', '--name',
-                             'test/test_project2')
+        self._run_gerrit_cli('create-project', 'test/test_project2',
+                             '--empty-commit')
         project2_uri = self.project_uri.replace('test/test_project',
                                                 'test/test_project2')
         self._run_git('fetch', project2_uri, 'HEAD')
@@ -472,6 +574,15 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
     def test_git_review_F_R(self):
         self.assertRaises(Exception, self._run_git_review, '-F', '-R')
 
+    def test_git_review_no_thin(self):
+        """Test git-review --no-thin."""
+        self._run_git_review('-s')
+
+        # Push with --no-thin set. We can't really introspect the packs
+        # that were sent so we infer success as the command not failing.
+        self._simple_change('test file modified', 'test commit message')
+        self._run_git_review('--no-thin')
+
     def test_config_instead_of_honored(self):
         self.set_remote('test_project_url')
 
